#!perl -w
use Test::More;
use strict;
use DE_EPAGES::Core::API::Html qw(
    BalanceHTMLTags
    RemoveHtmlTags
);

plan tests => 4;

# BalanceHTMLTags
my $SourceText  = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
my $GoalText    = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1></p><p>dfasdasdasdaasasfasfs</p>asdasda';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags headings and paragraphs');
$SourceText  = '<p><table><tr><td>test</tr><tr><td><div><strong><!-- pagebreak -->';
$GoalText    = '<p><table><tr><td>test</td></tr><tr><td><div><strong><!-- pagebreak --></strong></div></td></tr></table></p>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags malformed table');
$SourceText  = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak -->';
$GoalText    = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak --></td></tr></tbody></table></a></span></i></p>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags complex from tinymce');

# RemoveHtmlTags
$SourceText  = <<'END_TEXT';
    <p>
      <h1> <span color="#333311">headline</span> <strong><!-- pagebreak --></strong></h1>
    <p>text1<br>text 2<hr />&nbsp;&nbsp;text3</p>
    <script language="JScript">
      document.all.obsolete = 1;
    </script>
    <STYLE>
      document.all.obsolete = 1;
    </STYLE>
    fin.
END_TEXT
$GoalText = <<'END_TEXT';
headline
text1
text 2
  text3
fin.
END_TEXT
is( RemoveHtmlTags($SourceText)."\n", $GoalText, 'RemoveHtmlTags' );
