/*
	Copyright (c) 2010, ePages GmbH
	All Rights Reserved.

	epages.widget.FormElements $Revision: 1.19.2.1 $

*/

dojo.provide('epages.widget.FormElements');

dojo.require('dojo.date.locale');
dojo.require('dojo.regexp');
dojo.require('epages.epages');
dojo.require('epages.event');
dojo.require('epages.localize');
dojo.require('epages.browser');
dojo.require('epages.mimetype');
dojo.require('epages.io.translation');
dojo.require('epages.lang.hash');
dojo.require('epages.widget.Form');

(function(dj){

	var windowFormWdgt = new epages.widget.Form( {}, document.createElement('form') );

	dj.declare('epages.widget.FormBase', [dijit._Widget], {

	// VARS
		wrapNode		: null,				// node of wrap which wrapped the domNode
		formNode		: null,				// node of parent form
		_type				: 'text',			// type of formElement
		_attr				: {},					// attributes of formElement
		_handles		: null,				// array of handles which will connect on this._connectHandles

		_translation	: epages.io.dictionary.translation,


	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this._attr = {};
			this._handles = {};
			this._connects = {};
			// get type
			if( this.domNode.nodeName.match(/^(TEXTAREA|SELECT)$/) ){
				this._attr.type = this.domNode.nodeName.toLowerCase();
			}
			else {
				this._attr.type = dj.attr(this.domNode,'type');
			}
			if( this._attr.type == 'datetime-local' ){
				this._attr.type = 'datetime';
			}
			if( !this._attr.type ){
				var match = (this.domNode.outerHTML).match(/[<\s]{1}type="?([A-Za-z]+)"?[>\s\/]{1}/);
				this._attr.type = match ? match[1] : 'text';
			}

			// get attributes
			var ATTR = ['autofocus','form','formaction','formmethod','formnovalidate','id','required','pattern','min','max','name','accept','checked','disabled','list','maxlength','multiple','placeholder','step','value'];

			for( var i=0,l=ATTR.length ; i<l ; i++ ){
				var name = ATTR[i];
				var value = dj.attr(this.domNode,name);
				// there is a bug in dojo.attr => use native javascript for this attribute
				if( name == 'multiple' ){
					value = this.domNode.getAttribute('multiple') === 'true' ? true : false;
				}
				// parse min, max & step to number
				if( name.match(/^(max|min|step)$/) ){
					value = epages.localize.str2number(value);
					if( value !== null ){
						this._attr[name] = value;
					}
				}
				// is defined
				else if( value !== undefined ){
					this._attr[name] = value;
				}
			}
			this._attr.id = this.id;
			this._attr.valide = dj.hasClass(this.domNode,'DialogError') ? false : true;

			// replace domNode for type compatibility
			if( this._attr.type.match(/^(color|date|datetime|email|month|number|search|tel|time|url|week)$/) ){
				var node = dj.clone(this.domNode);
				if( !(epages.Browser.engine == 'MSIE' && epages.Browser.version <= 8) ){
					dj.attr( node , 'type' , 'text' );
				}
				dj.place( node , this.domNode , 'after' );
				node.parentNode.removeChild(this.domNode);
				this.domNode = node;
			}

			// if element hidden by style copy to wrap node
			var style = '';
			var originalStyle = this.domNode.getAttribute('style') || '';
			if(originalStyle !== '') {
				// Start IE Hack
				if( typeof originalStyle === 'object' ){
					var ret = [];
					for( var i in originalStyle ){
						ret.push( i + ":" + name[i] );
					}
					originalStyle = ret.join(';');
				}
				// End IE Hack
				originalStyle.replace(/(display:\s*[a-z]+\s*;?|visibility:\s*[a-z]+\s*;?)/gi, function(str,a){
					style = style+a;
					return "";
				});
			}

			// wrap domNode
			this.wrapNode = dj.create('label',{
				'class': 'EpForm ' + this._attr.type.substr(0,1).toUpperCase() + this._attr.type.substr(1) + ' ' + dj.attr(this.domNode,'class'),
				'style': style
			},this.domNode,'after');
			this.wrapNode.appendChild(this.domNode);

			// set formWidget
			var formNode = (this._attr.form ? dj.query('form[name='+this._attr.form+']') : [this.domNode.form])[0];
			if( !formNode ){
				this.formWdgt = windowFormWdgt;
			}
			else{
				this.formWdgt = $$( dj.attr(formNode,'widgetid') );
				if( !this.formWdgt ){
					this.formWdgt = new epages.widget.Form( {}, formNode );
				}
			}

			// fix to disable validation by browser
			dj.removeAttr( this.domNode, 'formmethod' );
			dj.removeAttr( this.domNode, 'formaction' );
			dj.removeAttr( this.domNode, 'max' );
			dj.removeAttr( this.domNode, 'min' );
			dj.removeAttr( this.domNode, 'pattern' );
			dj.removeAttr( this.domNode, 'required' );
			dj.removeAttr( this.domNode, 'step' );

			// add this formElement to his form
			this.formWdgt.addElem( this );

			this.addHandle( 'domNode.focus', this.domNode, 'focus', function() {
				dj.addClass(this.domNode,'Active');
				dj.addClass(this.wrapNode,'Active');
			});
			this.addHandle( 'domNode.blur', this.domNode, 'blur', function() {
				dj.removeClass(this.domNode,'Active');
				dj.removeClass(this.wrapNode,'Active');
			});

			if( this._attr.type.match(/^(checkbox|radio)$/) ){
				this.attr( 'startupValue', this._attr.checked ? this._attr.value : '' );
			}
			else{
				this.attr( 'startupValue', this._attr.value );
			}

			// set focus
			if( this._attr.autofocus == true ){
				this._domNode.focus();
			}

			// set the id also
			this.domNode.setAttribute( 'id', this.id );
		},

		// init the readonly and disbaled status
		postAfterCreate: function(){
			this.attr( 'readonly', this._attr.readonly || false );
			this.attr( 'disabled', this._attr.disabled || false );
		},

		// METHODS
		style: function( node, name, value ){
			if( !node.nodeName && !this[node] ){
				value = name;
				name = node;
				node = this.domNode;
			}
			node = node.nodeName || this[node];

			if( typeof name == 'object' ){
				dj.style( node, name );
				return this;
			}
			else if( value == undefined ){
				return dj.style( node, name );
			}
			else{
				dj.style( node, name, value );
				return this;
			}
		},
		attr: function( name, value ){
			if( typeof name == 'object' ){
				for( var i in name ){
					this.attr( i, name[i] );
				}
				return this;
			}
			else {
				var methodName = '_attr'+name.substr(0,1).toUpperCase()+name.substr(1);
				if( value === undefined ){
					return this[methodName] ? this[methodName]() : this._attr[name];
				}
				else {
					return this._attr[name] = this[methodName] ? this[methodName](value) : value;
				}
			}
		},
		_attrMin: function( value ){
			return value ? epages.localize.str2number(value) : this._attr.min;
		},
		_attrMax: function( value ){
			return value ? epages.localize.str2number(value) : this._attr.max;
		},
		_attrStep: function( value ){
			return value ? epages.localize.str2number(value) : this._attr.step;
		},
		_attrType: function( value ){
			// value will be ignored, type can not modify
			return this._attr.type;
		},
		// set/get formElement multiple status
		_attrValue: function( value ){
			if( value ){
				this.domNode.setAttribute('value', value);
				return value;
			}
			else {
				return this._attr.value || '';
			}
		},
		// set/get formElement disabled status
		_attrForm: function( value ){
			if( value || value == '' ){
				var formNodeList;
				if( value == '' ){
					formNodeList = [this.domNodeForm];
				}
				else {
					formNodeList = dj.query('form[name='+value+']') || [];
				}
				if( formNodeList[0] ){
					this.formWdgt.removeElem( this.id );
					this.formWdgt = $$( dj.attr(formNodeList[0], 'widgetid') );
					if( !formWdgt ){
						this.formWdgt = windowFormWdgt;
						this.formWdgt.addElem( this );
					}
					this._attr.form = value;
				}
				return this._attr.form || '';
			}
			else {
				return this._attr.form;
			}
		},
		// set/get formElement disabled status
		_attrDisabled: function( bool ){
			if( bool === true ){
				dj.attr( this.domNode , 'disabled' , true );
				dj.addClass( this.wrapNode , 'Disabled' );

				this._disconnectHandles();
				return bool;
			}
			else if( bool === false ){
				dj.removeAttr( this.domNode , 'disabled' );
				dj.removeClass( this.wrapNode , 'Disabled' );

				if( !this._attr.readonly ){
					this._disconnectHandles();
					this._connectHandles();
				}
				return bool;
			}
			else {
				return this._attr.disabled || false;
			}
		},
		// set/get formElement readonly status
		_attrReadonly: function( bool ){
			if( bool === true ){
				dj.attr( this.domNode , 'readonly', true );
				dj.addClass( this.wrapNode , 'Readonly' );

				this._disconnectHandles();
				return bool;
			}
			else if( bool === false )  {
				dj.removeAttr( this.domNode , 'readonly' );
				dj.removeClass( this.wrapNode , 'Readonly' );

				if( !this.attr('disabled') ){
					this._disconnectHandles();
					this._connectHandles();
				}
				return bool;
			}
			else {
				return this._attr.readonly || false;
			}
		},
		// add handle with id
		addHandle: function( id, node, event, fn  ) {
			if( this._connects[id] ) {
				this.removeHandle( id );
			}
			this._handles[id] = [ typeof node === 'string' ? this[node] : node, event, fn ];

			return this;
		},
		// remove handle by id
		removeHandle: function( id ) {
			if( this._connects[id] ) {
				dj.disconnect( this._connects[id] );
			}
			//delete( this._handles[id] );

			return this;
		},
		// connect handle by id
		connectHandle: function( id ) {
			if( this._connects[id] ) {
				this.removeHandle( id );
			}
			if( this._handles[id] ) {
				var _handle = this._handles[id];
				this._connects[id] = dj.connect( _handle[0], _handle[1], this, _handle[2] );
			}
			return this;
		},
		// disconnect handles by id
		disconnectHandle: function( id ) {
			dj.disconnect( this._connects[id] );

			return this;
		},
		// connect all handles
		_connectHandles: function(){
			var _handles = this._handles;
			for( var i in _handles ){
				this.connectHandle( i );
			}
		},
		// disconnect all handles
		_disconnectHandles: function(){
			var _handles = this._handles;
			for( var i in _handles ){
				this.disconnectHandle( i );
			}
		},
		// replace pseudo pattern with regexp pattern
		_replacePattern: function(){
			if( this._attr.pattern ){
				this._attr.pattern = this._attr.pattern.replace(/(\[\:[a-z\-]+\:\])/,function( t, s ){
					return epages.localize.getPattern( s.substr(2,s.length-4) );
				} );
			}
		},
		// validate formElement
		validate: function(){
			var valide = true;

			if( !this.attr('formnovalidate') ){
				valide = this._validate();
				var maxL = Math.min( this.attr('maxlength') || epages.vars.MaxTextLength, epages.vars.MaxTextLength );
				valide =  maxL && maxL<(dj.attr(this.domNode,'value')+'').length ? false : valide;
				if( valide ){
					dj.removeClass( this.domNode , 'DialogError' );
					dj.removeClass( this.wrapNode , 'DialogError' );
				}
				else {
					dj.addClass( this.domNode , 'DialogError' );
					dj.addClass( this.wrapNode , 'DialogError' );
				}
				this.attr('valide', valide ? true : false );
				this.formWdgt.change();
			}
			else {
				this.attr('valide', valide ? true : false );
			}

			if( dj.attr( this.domNode,'value') !=  this.attr('startupValue') ){
				dj.addClass( this.domNode, 'Changed' );
				dj.addClass( this.wrapNode, 'Changed' );
			}
			else{
				dj.removeClass( this.domNode, 'Changed' );
				dj.removeClass( this.wrapNode, 'Changed' );
			}

			return valide;
		},
		// placeholder for validate function
		_validate: function(){
			return true;
		},
		// destroy a widget
		destroy: function(){
			this.inherited('destroy',arguments);

			this._disconnectHandles();
			this.formWdgt.removeElem( this.id );
		}

	});


	// WIDGET FormButton
	dj.declare('epages.widget.FormButton', [epages.widget.FormBase], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.postAfterCreate();
		}

	});


	// WIDGET FormFile
	dj.declare('epages.widget.FormFile', [epages.widget.FormBase], {

		buttonNode: 		null,
		_domNode:				null,
		_handlesIndex: 	null,

		// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.buttonNode = dj.create('span',{
			  'class':''
			},this.domNode,'after');

			this.buttonNode.innerHTML = this._translation.get( this._attr.multiple ? 'FileUploadMultiple' : 'FileUploadSingle');

			this.postAfterCreate();
		},

		// METHODS
		// set/get formElement multiple status
		_attrAccept: function( value ){
			if( value ){
				this.domNode.setAttribute('accept',value);
				return value;
			}
			else {
				return this._attr.accept || '';
			}
		},
		// set/get formElement multiple status
		_attrMultiple: function( bool ){
			if( bool === true ){
				this.domNode.setAttribute('multiple',true);
				return bool;
			}
			else if( bool === false ){
				this.domNode.removeAttribute('multiple');
				return bool;
			}
			else {
				return this._attr.multiple || false;
			}
		},

		_attrAccept: function( value ){
			if( value ){
				var instance = new epages.Mimetype();
				this._attr.accept = value.replace(/([A-Za-z]+\/\*)/,dj.hitch(this,function( string, found ){
					var mimes = instance.mimesByGroup( found );
					if( mimes ) {
						return mimes.join(',');
					}
					else {
						return '';
					}
				}));
				dj.attr( this.domNode, 'accept', this._attr.accept );
				return this._attr.accept;
			}
			else {
				return this._attr.accept;
			}
		},

		// set/get formElement value
		_attrValue: function( value ){
			if( value == '' ){
				this._disconnectHandles();

				var arr = [];
				for( var i=0,iLength=this._handles.length ; i<iLength ; i++ ){
					if( this._handles[i][0] == this.domNode ) {
						arr.push(i);
					}
				}

				this._domNode = this.domNode;
				if( epages.Browser.engine == "Gecko"){
					this.domNode.setAttribute('value','');
				}
				else{
					this.domNode = dj.place( this.domNode.cloneNode(), this._domNode, 'replace' );
					var node = dj.clone(this.domNode);
				}

				for( var i in this._handles ) {
					if( this._handles[i][0] == this._domNode ) {
						this._handles[i][0] = this.domNode;
					}
				}

				for( var i=0,iLength=arr.length ; i<iLength ; i++ ){
					this._handles[arr[i]][0] = this.domNode;
				}

				this._connectHandles();
			}
			else {
				return this._attr.value || '';
			}
		}

	});


// WIDGET FormFileUploder
	dj.declare('epages.widget.FormFileUploader', [epages.widget.FormFile], {

		_uploader:		null,

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.addHandle( 'domNode.change.fileUploader', this.domNode, 'change', this._change  );
			this.addHandle( 'domNode.click.fileUploader', this.domNode, 'click', this._click  );

			this.attr({
				'sendaction':	dj.attr(this.domNode,'sendaction'),
				'deleteaction':	dj.attr(this.domNode,'deleteaction'),
				'finishaction':	dj.attr(this.domNode,'finishaction')
			});

			this.formWdgt.removeElem( this.id );

			this.postAfterCreate();
		},

	// METHODS
		// open the uploader if attr uploader is true
		_click:	function( event ){
			if( !this._uploader ){
				this._createUploader();
				this._uploader.hide();
			}
			else {
				event.preventDefault();
				this._uploader.show();
			}
		},
		// open the uploader if attr uploader is true
		_change: function(){
			if( !this._uploader ){
				this._createUploader();
			}
			this._uploader.show();
			this.attr('value', '');
			this._uploader.externalInput( this._domNode );
		},

		_createUploader: function(){
			dj.require('epages.widget.Uploader');

			this._uploader = new epages.widget.Uploader({
				options:{
					id: 						this.attr('id'),
					multiple : 			this.attr('multiple'),
					accept : 				this.attr('accept') || '*/*',
					sendaction : 		this.attr('sendaction'),
					deleteaction : 	this.attr('deleteaction'),
					finishaction : 	this.attr('finishaction')
				}
			}, dj.create('div'));
		}
	});


// WIDGET FormSelect
	dj.declare('epages.widget.FormSelect', [epages.widget.FormBase], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.addHandle( 'domNode.change.select', this.domNode, 'change', this.validate  );

			this.postAfterCreate();
		},

	// METHODS
		// set/get formElement required status (required cann't be true for type select
		_attrRequired: function(){
			this._attr.required = false;
			return false;
		}
	});


// WIDGET FormText
	dj.declare('epages.widget.FormText', [epages.widget.FormBase], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.addHandle( 'domNode.keyup.text', this.domNode 		, 'keyup' 	, this.validate );
			this.addHandle( 'domNode.blur.text',  this.domNode 		, 'blur' 		, this.validate );

			if( this.domNode.getAttribute("emptyValue") ){
				var domNode = this.domNode;
				var value = dj.attr( domNode, 'value' );
				var emptyValue = dj.attr( domNode, 'emptyValue' );

				if( !value || value == emptyValue ){
					dj.attr( domNode, 'value', emptyValue );
					dj.addClass( domNode, 'EmptyValue' );
				}

				dojo.connect( this.formWdgt.domNode, "submit", function() {
					domNode.value = domNode.value.replace(domNode.getAttribute('emptyValue'), '');
					dojo.removeClass(domNode, "EmptyValue");
				});
				this.addHandle( 'domNode.focus2.text', domNode, "focus", function() {
					domNode.value = domNode.value.replace(domNode.getAttribute('emptyValue'), '');
					dojo.removeClass(domNode, "EmptyValue");
				});
				this.addHandle( 'domNode.blur2.text', domNode, "blur", function() {
					if(domNode.value == "") {
						dojo.addClass(domNode, "EmptyValue");
						domNode.value = domNode.getAttribute("emptyValue");
					}
				});
	   }

			// replace domNode for type compatibility
			if( this._attr.type.match(/^(email|tel|url)$/) ){
				this._attr.pattern = epages.localize.getPattern( this._attr.type );
			}

			// handle mandatory
			if( dj.hasClass( this.domNode, 'Mandatory' ) && !dj.attr( this.domNode, 'value' ) ){
				dj.addClass( this.domNode, 'Unfilled' );
			}

			this.postAfterCreate();
		},

	// METHODS
		_validate: function(){
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;
			if( value == '' && !this._attr.required ){
				valide = true;
			}
			if( value == '' && this._attr.required ){
				valide = false;
			}
			else if( this._attr.pattern && value.match(new RegExp('^'+this._attr.pattern+'$')) ){
				valide = true;
			}
			else if( !this._attr.pattern ){
				valide = true;
			}
			if( dj.hasClass( this.domNode, 'Mandatory' ) && !dj.attr( this.domNode, 'value' ) ){
				dj.addClass( this.domNode, 'Unfilled' );
			}
			else{
				dj.removeClass( this.domNode, 'Unfilled' );
			}
			return valide;
		}
	});


// WIDGET FormCheck
	dj.declare('epages.widget.FormCheck', [epages.widget.FormBase], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.addHandle( 'domNode.change.check', this.domNode, 'change', this._change );
			if( epages.Browser.engine == 'MSIE' ){
				this.addHandle( 'domNode.click.check', this.domNode, 'click', this._ieChangeFix );
			}

			if( !this.domNode.value && !this.domNode.defaultValue ){
				this.domNode.defaultValue = "on";
			}

			this.attr( 'checked', this._attr.checked ? true : false );
			this.postAfterCreate();
		},

	// METHODS
		// set/get formElement checked status
		_attrChecked: function(bool){
			if(bool === true){
				if( this._attr.type == 'radio' ){
					var radios = this._getRadios();
					for( var i=0,l=radios.length ; i<l ; i++ ){
						radios[i].attr( 'checked', false );
						dojo.removeClass(radios[i].domNode,'DialogError');
						dojo.removeClass(radios[i].wrapNode,'DialogError');
					}
				}
				dj.attr( this.domNode, 'checked', true );
				dj.addClass( this.wrapNode, 'Checked' );
				return bool;
			}
			else if(bool === false){
				dj.attr( this.domNode, 'checked', false );
				dj.removeClass( this.wrapNode, 'Checked' );
				return bool;
			}
			else {
				return this._attr.checked;
			}
		},
		// a fix for the IE to fire the change action instantly
		_ieChangeFix: function(){
			this.domNode.blur();
		},
		// handle the change
		_change: function(){
			this.attr( 'checked', dj.attr( this.domNode , 'checked' ) ? true : false );
			this.validate();
		},
		// found all radio element with same name in this form and outside of this form which added to this form by the form attribute
		_getRadios: function(){
			var domNodeName = dj.attr(this.domNode,'name');
			var formNodeName = dj.attr(this.formWdgt.domNode,'name');

			var inForm;
			var exForm;
			if( this.formWdgt === windowFormWdgt ){
				inForm = dj.query('input[type=radio][name='+domNodeName+']',this.domNode.ownerDocument);
				exForm = [];
			}
			else{
				inForm = dj.query('input[type=radio][name='+domNodeName+']',this.formWdgt.domNode);
				exForm = dj.query('input[type=radio][name='+domNodeName+'][form='+formNodeName+']',document.body);
			}

			var radios = inForm.concat(exForm);
			var required = this.attr('required');

			var ret = [];
			for( var i=0,l=radios.length ; i<l ; i++ ){
				var wdgt = $$( dj.attr(radios[i],'widgetid') );
				if( wdgt ){
					ret.push( wdgt );
					required = wdgt.attr('required') || required;
					wdgt.attr('valide',true);
				}
			}
			this.attr('required',required);

			return ret;
		},

		_validate: function(){

			var value = dj.attr( this.domNode , 'value' );
			var valide = false;
			if( this._attr.type == 'checkbox' && ( (dj.attr(this.domNode,'checked') && this._attr.required) || !this._attr.required ) ){
				valide = true;
			}

			if( this._attr.type == 'radio' ){
				var radios = this._getRadios();
			}

			if( this._attr.type == 'radio' && ( (this.attr('checked') && this.attr('required')) || !this.attr('required') ) ){
				valide = true;
			}
			else if( this._attr.type == 'radio' ){
				for( var i=0,l=radios.length ; i<l ; i++ ){
					valide = radios[i].attr('checked') || valide;
				}
			}
			return valide;
		}

	});


// WIDGET FormNumber
	dj.declare('epages.widget.FormNumber', [epages.widget.FormBase], {

		stepUpNode: 			null,			// node of the step up button
		stepDownNode: 		null,			// node of the step down button

		_stepCount:				null,			// counter of intervals
		_stepSpeed: 			null,			// interval time
		_stepInterval: 		null,			// interval of step action

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this._replacePattern();

			this.stepUpNode = dj.create('a',{
					'href':'javascript:;',
					'class':'StepUp'
				});

			this.stepDownNode = dj.create('a',{
					'href':'javascript:;',
					'class':'StepDown'
				});

			this.wrapNode.appendChild(this.stepUpNode);
			this.wrapNode.appendChild(this.stepDownNode);

			this.addHandle( 'domNode.keyup.number',          this.domNode 			, 'keyup' 			, this.validate  );
			this.addHandle( 'domNode.blur.number',           this.domNode 			, 'blur' 				, this.validate  );

			this.addHandle( 'stepUpNode.mousedown.number',   this.stepUpNode 		, 'mousedown' 	, function(event){ event.preventDefault(); this._stepStart(+1); }  );
			this.addHandle( 'stepUpNode.mouseup.number',     this.stepUpNode 		, 'mouseup'   	, this._stepStop  );
			this.addHandle( 'stepUpNode.mouseout.number',    this.stepUpNode 		, 'mouseout'  	, this._stepStop  );

			this.addHandle( 'stepDownNode.mousedown.number', this.stepDownNode 	, 'mousedown' 	, function(event){ event.preventDefault(); this._stepStart(-1); }  );
			this.addHandle( 'stepDownNode.mouseup.number',   this.stepDownNode 	, 'mouseup' 		, this._stepStop  );
			this.addHandle( 'stepDownNode.mouseout.number',  this.stepDownNode 	, 'mouseout' 		, this._stepStop  );

			// set pattern
			this._attr.pattern = this._attr.pattern || epages.localize.getPattern( 'number' );

			this.postAfterCreate();
		},

	// METHODS
		// handle the step
		_stepAction: function( inc ){
			this._stepCount++;
			this._stepSpeedHandle( inc );

			var value = dj.attr( this.domNode , 'value' );
			var ret;
			var step = inc * (this._attr.step || 1);
			var attrMin = this._attr.min;
			var attrMax = this._attr.max;


			ret = epages.localize.str2number(value);
			ret = ret!==null ? ret : 0;
			var i = (ret+'').indexOf('.');
			var l = i>=0 ? (ret+'').substr(i+1).length : 0;
			ret = ret+step;
			ret = (attrMin && attrMin > ret.toFixed(l)) ? attrMin : (attrMax && attrMax < ret.toFixed(l)) ? attrMax : ret.toFixed(l);
			ret = epages.localize.number2str( ret );
			dj.attr( this.domNode , 'value' , ret );
		},
		// start the step action
		_stepStart: function( inc ){
			this._stepCount = 0;
			this._stepSpeed = 250;

			this._stepAction(inc);
			this._stepInterval = window.setInterval( dj.hitch(this,function(){
				this._stepAction(inc);
			}), this._stepSpeed );
		},
		// stop the step action
		_stepStop: function(){
			window.clearInterval( this._stepInterval );
			epages.event.fire(this.domNode,'blur');
		},
		// handle the speed of the step action
		_stepSpeedHandle: function( inc ){
			if( dj.indexOf([10,30,50],this._stepCount)>=0 ){
				this._stepSpeed = this._stepSpeed / 2;
				window.clearInterval( this._stepInterval );
				this._stepInterval = window.setInterval( dj.hitch(this,function(){
					this._stepAction(inc);
				}), this._stepSpeed );
			}
		},

		_validate: function(){
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;

			if( value=='' && !this._attr.required ){
				valide = true;
			}
			else if( value.match(new RegExp('^'+this._attr.pattern+'$')) ){
				var val = epages.localize.str2number(value);
				var attrMin = this._attr.min;
				var attrMax = this._attr.max;
				valide = (attrMin && attrMin > val) ? false : (attrMax && attrMax < val) ? false : true;

			}
			return valide;
		}

	});


// WIDGET FormDate
	dj.declare('epages.widget.FormDate', [epages.widget.FormNumber], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			// set pattern by type
			this._attr.pattern = epages.localize.getPattern( this._attr.type );
		},

	// METHODS
		_stepAction: function( inc ){
			this._stepCount++;
			this._stepSpeedHandle( inc );

			var value = dj.attr( this.domNode , 'value' );
			var ret;
			var attrStep = this._attr.step || 1;
			var attrMin = this._attr.min;
			var attrMax = this._attr.max;

			var dateObj = epages.localize.date2obj( value , this._attr.type ) || new Date();
			switch( this._attr.type ){
				case 'datetime':
					dateObj.setHours( dateObj.getHours() + (inc * attrStep) );
				break;
				case 'month':
					dateObj.setMonth( dateObj.getMonth() + (inc * attrStep) );
				break;
				case 'time':
					dateObj.setHours( dateObj.getHours() + (inc * attrStep) );
				break;
				case 'week':
					dateObj.setDate( dateObj.getDate() + (inc * attrStep * 7) );
				break;
				case 'date':
				default:
					dateObj.setDate( dateObj.getDate() + (inc * attrStep) );
				break;
			}
			var dateTime = dateObj.getTime()/1000;
				dateTime = (attrMin && attrMin > dateTime) ? attrMin : (attrMax && attrMax < dateTime) ? attrMax : dateTime;

			dateObj.setTime( dateTime*1000 );

			ret = epages.localize.obj2date( dateObj , this._attr.type );

			dj.attr( this.domNode , 'value' , ret );
		},

		_validate: function(){
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;

			if( value=='' && !this._attr.required ){
				valide = true;
			}
			else if( value.match(new RegExp('^'+this._attr.pattern+'$')) ){
				var dateTime = epages.localize.date2obj( value , this._attr.type ).getTime()/1000;
				var attrMin = this._attr.min;
				var attrMax = this._attr.max;

				valide = (attrMin && attrMin > dateTime) ? false : (attrMax && attrMax < dateTime) ? false : true;

			}
			return valide;
		}

	});

})(dojo);