/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.base64
 *
 * @revision		$Revision: 1.2.2.2 $
 */

jQuery.plugin( 'ep.base64', ['ep.core','ep.utf8','jQuery.json'], function( $, undefined ){

	var keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";

	ep.base64Encode = function( str, noJSON ){
		str = ep.utf8Encode( noJSON ? str : $.toJSON(str) );

		//check if browser function btoa base64ToASCII is defined
		if( $.isFunction(window.btoa) ){
			return window.btoa(str);
		}

		var output = "",
			chr1, chr2, chr3, enc1, enc2, enc3, enc4,
			i = 0,
			strLength = str.length;

		while (i < strLength) {
			chr1 = str.charCodeAt(i);
			i++;
			chr2 = str.charCodeAt(i);
			i++;
			chr3 = str.charCodeAt(i);
			i++;

			enc1 = chr1 >> 2;
			enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
			enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
			enc4 = chr3 & 63;

			if (isNaN(chr2)) {
				enc3 = enc4 = 64;
			} else if (isNaN(chr3)) {
				enc4 = 64;
			}

			output = output +
			keyStr.charAt(enc1) + keyStr.charAt(enc2) +
			keyStr.charAt(enc3) + keyStr.charAt(enc4);

		}

		return output;
	};

	ep.base64Decode = function( str, noJSON ){

		var output = "",
			chr1, chr2, chr3, enc1, enc2, enc3, enc4,
			i = 0;

		str = str.replace(/[^A-Za-z0-9\+\/\=]/g, "");
		//check if browser function atob ASCIItoBase64 is defined
		if (typeof(window.atob) === 'function') {
			output = window.atob(str);
		}
		else {
			var strLength = str.length;
			while (i < strLength) {

				enc1 = keyStr.indexOf(str.charAt(i));
				i++;
				enc2 = keyStr.indexOf(str.charAt(i));
				i++;
				enc3 = keyStr.indexOf(str.charAt(i));
				i++;
				enc4 = keyStr.indexOf(str.charAt(i));
				i++;

				chr1 = (enc1 << 2) | (enc2 >> 4);
				chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
				chr3 = ((enc3 & 3) << 6) | enc4;

				output = output + String.fromCharCode(chr1);

				if (enc3 !== 64) {
					output = output + String.fromCharCode(chr2);
				}
				if (enc4 !== 64) {
					output = output + String.fromCharCode(chr3);
				}

			}
		}

		output = ep.utf8Decode(output);

		return noJSON ? output : $.parseJSON(output);
	};
});