/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.tooltip
 *
 * @revision		$Revision: 1.5.2.4 $
 */

jQuery.plugin( 'ep.ui.tooltip', ['ep.core', 'jQuery.ui.widget', 'ep.fn.contextOrientation'], function( $, undefined ){

	var EVENTS = {
			'focus': ['focus.uiTooltip click.uiTooltip','blur.uiTooltip'],
			'click': ['click.uiTooltip'],
			'hover': ['mouseenter.uiTooltip','mouseleave.uiTooltip']
		};

	$.widget( "ep.ui.Tooltip",/* my.parent.widget,*/ {

		options: {
			event:			'hover',
			context:		null,
			interactive:	false,
			orientation:	'',			// can be top,left,right,bottom,center,middle [empty = cursor]
			offsetAdjust:	[0,0],
			addClass:		'',
			showDelay:		200,
			hideDelay:		400
		},

		_create: function(){
			this.elem = this.element
				.subclass(ep)
				.removeClass('HideElement')
				.removeAttr('style');

			// add context + bind events
			this.context = $(this.options.context);

			// add wrapper around tooltip content
			this.wrap = ep('<div>')
				.addClass('ep-uiTooltip ' + this.options.addClass)
				.attr( 'style', this.elem.attr('style') || '' )
				.appendTo('body')
				.append(this.elem)
				.hide();
		},

		_init: function(){
			this.wrap.unbind('.uiTooltip');
			this.context.unbind('.uiTooltip');

			// dojo.connect(this.contextNode,'onclick',this,'_onClick');
			this.wrap
				.bind( 'mouseenter.uiTooltip', $.proxy(this,'_enter') )
				.bind( 'mouseleave.uiTooltip', $.proxy(this,'_leave') );

			var e = EVENTS[this.options.event];
			if( e[0] ){
				this.context.bind( e[0], $.proxy(this,'show') );
			}
			if( e[1] ){
				this.context.bind( e[1], $.proxy(this,'_hide') );
			}
		},

		_enter: function( event ){
			if( this.options.interactive ){
				this.cursorInside = true;
				this.wrap.stop(true,true);
			}
		},

		_leave: function( event ){
			this.cursorInside = false;
			this.hide();
		},

		show: function( event, callback ){
			var o = this.options
				context = this.context,
				orient = o.orientation,
				add = o.offsetAdjust;

			if( event instanceof $.Event && !orient && o.event!=='focus' ){
				context = event;
			}
			else if( $.isArray(event) ){
				context = new $.Event({
					pageX: event[0],
					pageY: event[1]
				});
			}

			this.wrap
				.contextOrientation(context,orient,add)
				.stop(true,true)
				.delay(o.showDelay)
				.fadeIn( 'normal', callback );

			this.elem.triggerHandler('show');

			$('html').bind( 'mouseup', $.proxy(this,'_hide') );
		},

		_hide: function( event ){
			if(!this.cursorInside){
				this.hide();
			}
		},

		hide: function(callback){
			var o = this.options;

			this.wrap
				.stop(true,true)
				.delay( o.interactive ? o.hideDelay+400 : o.hideDelay )
				.fadeOut( 'normal', callback );
			$('html').unbind( 'mouseup', $.proxy(this,'_hide') );
		},

		destroy: function(){
			this.wrap.unbind('.uiTooltip');
			this.elem.unwrap();
			this.context.unbind('.uiTooltip');

			$('html').unbind( 'mouseup', $.proxy(this,'_hide') );

			this.Inherited(arguments);
		}

	});

});
