/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.uri
 *
 * @revision		$Revision: 1.5.2.3 $
 */

jQuery.plugin( 'ep.uri', 'ep.core', function( $, undefined ){

	var uriExpr = /^((?:(.+)\:\/\/([^\/]+))?((?:\/|[^\?#]).*?)?(\?.*?)?(#.*)?)$/,
		absExpr = /^((http|https|ftp)\:\/\/|\/)/g;

	ep.extend({
		uriSegment: function( obj ) {
			var ret;
			if( typeof obj === 'string' ){
				var parts = obj.match(uriExpr);

				ret = parts ? {
						'href':				parts[1] || '',
						'protocol':			parts[2] || '',
						'host':				parts[3] || '',
						'path':				parts[4] ? parts[4].replace(/\/?(.*)$/i,"$1") : '',
						'query':			parts[5] || '',
						'anchor':			parts[6] || '',
						'parameters':		parts[5] ? ep.uriQuery( parts[5] ) : {}
					} : {};
			}
			else if( typeof obj === 'object' ){
				ret = '';
				ret += obj.protocol ? obj.protocol + '://' : '';
				ret += obj.host ? obj.host + '/' : '';
				ret += obj.path ? obj.path.replace(/\/?(.*)$/i,"$1") : '';

				if( obj.parameters ){
					var queryString = ep.uriQuery(obj.parameters);
					ret += queryString ? '?'+queryString : '';
				}
				else{
					ret += obj.query || '';
				}

				ret += obj.anchor || '';
			}
			return ret;
		},
		uriQuery: function( obj ){
			var ret;
			if( typeof obj === 'string' ){
				ret = {};
				var pairs = obj.replace(/^(\?)/,'').split('&');
				for (var i=0; i<pairs.length; i++) {
					var pair = pairs[i].split('='),
						accessors = [],
						name = decodeURIComponent(pair[0]),
						value = decodeURIComponent(pair[1] || '');

					name = name.replace( /\[([^\]]*)\]/g, function( k, acc ){ accessors.push(acc); return ""; } );
					accessors.unshift(name);

					var tmp = ret,
						acc;
					for( var j=0,jLength=accessors.length ; j<jLength-1 ; j++ ){
						var nextAcc = accessors[j+1];
						acc = accessors[j];

						if (!tmp[acc]) {
							if( (nextAcc === "") || (/^[0-9]+$/.test(nextAcc)) ){
								tmp[acc] = [];
							}
							else{
								tmp[acc] = {};
							}
						}
						tmp = tmp[acc];
					}
					acc = accessors[accessors.length-1];
					if( acc === "" ){
						tmp.push(value);
					}
					else{
						tmp[acc] = value;
					}
				}
			}
			else if( obj ){
				ret = $.param(obj);
			}
			return ret;
		},
		uriIsAbsolute: function( url ){
			return absExpr.test(url);
		},
		uriToAbsolute: function( url, baseUrl ){
			return (baseUrl || ep.config.baseUrl).replace(/(\/)$/,'') + "/" + ep.uriToRelative( url, baseUrl );
		},
		uriToRelative: function( url, baseUrl ){
			var srcUrl = url;
			url = ep.uriSegment( url );
			baseUrl = ep.uriSegment( baseUrl || ep.config.baseUrl );

			if ( (url.protocol && url.protocol !== baseUrl.protocol) || (url.host && url.host !== baseUrl.host) ){
				return srcUrl;
			}

			var parts = ep.escapeExpStr( baseUrl.path.replace(/^\/?(.*)$/i,"$1").replace(/(.*)\/$/i,"$1") ).split('\\/'),
				exprArr = [];

			for( var i=parts.length-1 ; i>=0 ; i-- ){
				exprArr.unshift(parts[i]);

				var expr = new RegExp( '^(' + exprArr.join('\\/') + ')', '' );

				if( expr.test(url.path) ){
					url.path = url.path.replace(expr,'').replace(/^(\/)/,'');
					break;
				}
			}

			return ep.uriSegment({
				'path':				url.path,
				'query':				url.query,
				'anchor':			url.anchor,
				'parameters':			url.params
			});
		}
	});

});