/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @core			jQuery JavaScript Library v1.4.4 + SubClass pre build-in from jQuery v1.5b
 *
 * @desc			jQuery.core.sdk includes
 *					- .subclass			In chaining subclass switch
 *					- jQuery.config		Configuration vars
 *					- jQuery.debug		Base property for debug tools
 *					- jQuery.plugin		Dependencies controller
 *					- jQuery.provide	Dependencies controller
 *					- jQuery.ready		Dependencies controller
 *					- jQuery.ready		DOM ready
 */

(function( jQuery, undefined ){

	jQuery.subclass = jQuery.sub;

	// .subclass() provide subclass switch as prototype method
	jQuery.fn.subclass = jQuery.fn.sub = function( jQuerySubclass ){
		return jQuerySubclass( this );
	};

	// jQuery.config
	var metaConfig = jQuery('meta[name=jQueryConfig]').attr('content'),
		config = jQuery.extend( true, {}, window.jQueryConfig, (metaConfig?eval('('+metaConfig+')'):undefined) );

	// run config for ajax
	jQuery.extend( jQuery.ajaxSettings, config.ajax );

	// run config for noConflict mode
	if( config.noConflict ){
		var alias = jQuery.noConflict();
		if( typeof config.noConflict === 'string' )
			window[ config.noConflict ] = alias;
	}

		// save original method
	var	fn_ready = jQuery.fn.ready,
		// save original method
		ready = jQuery.ready,

		// exclude from load via ajax
		exclExp = /^plugin:jQuery\.(ready|provide)\./,

		// count for ready calls
		readyCount = 0,

		// data stacks
		loadStack = [],
		waitStack = [],

		sortStack = [],			// type:dependName
		dependStack = {},		// type:dependName: [ depend, ... ]
		callStack = {},			// type:dependName: callback

		// controller
		controller = function(){

			// loop in stack
			for( var i=sortStack.length-1 ; i>=0 ; i-- ){
				var name = sortStack[ i ],
					depends = dependStack[ name ],
					depReady = true;

				// maybe name is undefined because dependReady modified sortStack
				if( !name || !depends )
					continue;

				// loop dependencies
				jQuery.each( dependStack[ name ], function( dep ){
					var depPart = dep.split( ':' ),
						depType = depPart.shift(),
						depName = depPart.join(':'),
						depHandle = dependHandle[depType];

					// if type not configured
					if( !depHandle ){
						depReady = false;
						return;
					}

					// is loaded or ready or currently loading
					if( depHandle.isReady(depName) )
						return;

					depReady = false;
					// is loaded or ready or currently loading
					if( depHandle.isLoad(depName) || jQuery.inArray(dep,loadStack)>=0 )
						return;

					// if not waiting
					if( jQuery.inArray( dep, waitStack )<0 && !exclExp.test(dep) )
						waitStack.push( dep );

					// if free capacity in loadstack
					if( loadStack.length < dependSettings.requestLimit && waitStack.length ){
						var add = waitStack.shift(),
							addPart = add.split(':'),
							addType = addPart.shift(),
							addName = addPart.join(':');

						loadStack.push( add );
						jQuery.ajax( dependHandle[addType].ajax(addName) );
					}
					else{
						return false;
					}
				});

				if( depReady ){
					dependReady( name, 'plugin', depends );
				}
			}
		},
		// add dependencies to control
		add2control = function( name, depends, callback ){
			var type = jQuery.type(depends);

			dependLoad( name, 'plugin' );

			// if no dependencies
			if( type==='function' ){
				callStack[ name ] = depends;
				dependReady( name, 'plugin', {} );
				return;
			}

			// if only plugins dependent
			if( (/(string|array)/).test(type) )
				depends = { plugin: depends };

			// add to stacks
			sortStack.push(name);
			callStack[ name ] = callback;
			var depStack = dependStack[ name ] = {};
			// generate adds for depend Stack
			jQuery.each( depends, function( depType, deps ){

				var prefix = depType==='script' ? 'plugin:jQuery.provide.' : depType+':';

				jQuery.each( typeof(deps)==='string' ? [deps] : deps, function( i, depName ){

					depStack[ prefix+depName ] = true;

				});
			});

			// run dependencies controller
			controller();
		};

	jQuery.extend({
		// jQuery.config
		config: config,
		// jQuery.debug = base property for debug tools
		debug: {},
		// jQuery.pluginSettings
		dependSettings: jQuery.extend(
			{
				minify:			false,
				jsonp:			false,
				requestLimit:	10,
				pluginRoot:		'.'
			},
			config.depend
		),
		// jQuery.pluginHandle
		dependHandle: {
			plugin: {
				ajax: function( name ){
					var ajax = {
							url:			dependSettings.pluginRoot+'/'+name.replace(/\./g,'/')+(dependSettings.minify?'.min':'')+'.js',
							dataType:		'script',
							type:			'get',
							asnyc:			true
						};

					return dependSettings.jsonp ?
						$.extend(ajax,{
							dataType:		'jsonp',
							jsonp:			str(dependSettings.jsonp,true) || undefined,
							jsonpCallback:	'jsonp.dependHandle.plugin('+name+')'
						}) :
						ajax;
				},
				isLoad: function( name ){
					return !!this._load[ name ];
				},
				isReady: function( name ){
					return !!this._ready[ name ];
				},
				onReady: function( name , data ){
					this._ready[ name ] = data;

						// get callback before delete
					var callback = callStack[ name ],
						// remove from data Stack
						index = jQuery.inArray( name, sortStack );

					if( index >= 0 )
						sortStack.splice( index, 1 );

					delete( callStack[ name ] );
					delete( dependStack[ name ] );

					callback( jQuery );
				},
				onLoad: function( name /*, data */ ){
					this._load[ name ] = true;
				},
				_load: {

				},
				_ready: {
					'jQuery.core.original': {},
					'jQuery.core.sdk': {}
				}
			}
		},
		// jQuery.dependLoad
		dependLoad: function( name, type, data ){
			var index = jQuery.inArray( type+':'+name, loadStack );
			if( index >= 0 )
				loadStack.splice( index, 1 );

			// call onLoad
			dependHandle[ type ].onLoad( name, data );
		},
		// jQuery.dependReady
		dependReady: function( name, type, data ){
			// call onReady
			dependHandle[ type ].onReady( name, data );

			controller();
		},
		// jQuery.ready (call a function on dependencies ready)
		ready: function( depends, callback ){
			var type = jQuery.type(depends);
			if( (/^(string|object|array)$/).test(type) )
				add2control( 'jQuery.ready.'+(readyCount++), depends, callback );
			else if( type === 'function' )
				fn_ready(depends);
			else
				return ready(depends);
		},
		provide: function( name, depends, callback ){
			add2control( 'jQuery.provide.'+name, depends, callback );
		},
		// jQuery.plugin
		plugin: function( name, depends, callback ){
			add2control( name, depends, callback );
		}
	});

		// [short scope => local alias]
	var dependSettings = jQuery.dependSettings,
		dependHandle = jQuery.dependHandle,
		dependLoad = jQuery.dependLoad,
		dependReady = jQuery.dependReady;

})(jQuery);