/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
  * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.event.special.touch
 */

jQuery.plugin( 'jQuery.event.special.touch', 'jQuery.event.fix', function( $, undefined ){

	var html = $('html'),
		touchstart = function(originalBind){
			var ret = touchstart[originalBind.id] = function(event){
					if( originalBind.type==='touchdrag' && event.touches.length === 1 && event.originalTarget===originalBind.target){
						html.bind( 'touchmove', touchmove(originalBind) )
							.bind( 'touchend', touchend(originalBind) )
							.bind( 'touchcancel', touchcancel(originalBind) );

						touchmove[originalBind.id].pageX = event.pageX;
						touchmove[originalBind.id].pageY = event.pageY;
					}
					else if( originalBind.type==='touchzoom' &&  event.touches.length === 2 && event.touches[0].target===originalBind.target && event.touches[1].target===originalBind.target){
						html.bind( 'touchmove', touchmove(originalBind) )
							.bind( 'touchend', touchend(originalBind) )
							.bind( 'touchcancel', touchcancel(originalBind) );

						var touches = event.touches,
							deltaX = Math.abs( touches[0].pageX - touches[1].pageX ),
							deltaY = Math.abs( touches[0].pageY - touches[1].pageY );
							delta  = Math.sqrt( Math.pow(deltaX,2) + Math.pow(deltaY,2) );

						touchmove[originalBind.id].deltaX = deltaX;
						touchmove[originalBind.id].deltaY = deltaY;
						touchmove[originalBind.id].delta  = delta;
					}
				};
			return ret;
		},
		touchmove = function(originalBind){
			var ret = touchmove[originalBind.id] = function(event){
					event.type = originalBind.type;
					event.target = originalBind.target;
					event.data = originalBind.data;

					if( originalBind.type==='touchdrag' ){

						event.dragDeltaX = event.pageX - touchmove[originalBind.id].pageX;
						event.dragDeltaY = event.pageY - touchmove[originalBind.id].pageY;
						event.dragDelta  = event.dragDeltaY || event.dragDeltaX;

						touchmove[originalBind.id].pageX = event.pageX;
						touchmove[originalBind.id].pageY = event.pageY;

						return originalBind.originalHandler.apply(originalBind.target,arguments);
					}
					else if( originalBind.type==='touchzoom' &&  event.touches.length >= 2 ){
						var touches = event.touches,
							deltaX = Math.abs( touches[0].pageX - touches[1].pageX ),
							deltaY = Math.abs( touches[0].pageY - touches[1].pageY );
							delta  = Math.sqrt( Math.pow(deltaX,2) + Math.pow(deltaY,2) );

						event.zoomDeltaX = deltaX - touchmove[originalBind.id].deltaX;
						event.zoomDeltaY = deltaY - touchmove[originalBind.id].deltaY;
						event.zoomDelta  = delta  - touchmove[originalBind.id].delta;

						touchmove[originalBind.id].deltaX = deltaX;
						touchmove[originalBind.id].deltaY = deltaY;
						touchmove[originalBind.id].delta  = delta;

						return originalBind.originalHandler.apply(originalBind.target,arguments);
					}
				};
			return ret;
		},
		touchend = function(originalBind){
			var ret = touchend[originalBind.id] = function(event){
					html.unbind( 'touchmove', touchmove[originalBind.id] )
						.unbind( 'touchend', touchend[originalBind.id] )
						.unbind( 'touchcancel', touchcancel[originalBind.id] );

					delete(touchmove[originalBind.id]);
					delete(touchend[originalBind.id]);
					delete(touchcancel[originalBind.id]);
				};
			return ret;
		},
		touchcancel = touchend;

	// touchdrag, touchzoom
	$.each(['touchdrag', 'touchzoom'], function( i, name ) {
		$.event.special[ name ] = {
			add: function( bind ) {
				bind.target = this;
				bind.originalHandler = bind.handler;

				html.bind( 'touchstart', touchstart(bind) );
			},
			remove: function( unbind ) {
				html.unbind( 'touchstart', touchstart[unbind.id] );
				html.unbind( 'touchmove', touchmove[unbind.id] );

				delete(touchstart[unbind.id]);
				delete(touchmove[unbind.id]);
			}
		};
	});

});