/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.fn.form
 */

jQuery.plugin( 'jQuery.fn.form', 'jQuery.expr', function( $, undefined ){

	var _form = document.createElement('form'),

		// clear value or uncheck or unselect elements
		clearValues = function( elem ){
			elem.filter(':checked')
					.removeAttr('checked')
				.end()
				.not('select,:radio,:checkbox')
					.val('')
				.end()
				.find('options:selected')
					.removeAttr('selected');
		};

	$.fn.extend({
		// find all associated inputs to the first matched (default selector ":radio")
		formGroup: function( selector ){
			selector = selector || ':radio';

			var elem = this.filter( selector + ':first' )[0] || {},
				name = elem.name,
				elems = [];

			if ( name ) {
				var form = elem.form,
					owner = elem.ownerDocument;

				$.each( selector = selector.split(','), function( i, sel ){
					selector[i] = ( sel + "[name='" + name + "']" );
				});
				selector = selector.join(',');

				if ( form ) {
					elems = $(form)
						.formInput()
						.filter(selector)
						.get();
				}
				else if( owner ){
					elems = $( ':input', owner )
						.filter(selector)
						.filter(function() {
							return !this.form;
						})
						.get();
				}
			}

			return this.pushStack( elems );
		},
		// find all inputs of the the first matched form
		formInput: function(){
			return this.pushStack( (this.filter('form:first')[0] || {}).elements );
		},
		// reset all selected forms or formInputs
		formReset: function(){
			var stack = [],
				form = this
					.filter('form:first'),
				elems = this
					.filter(':input')
					.each(function(){
						var elem = $(this),
							spacer = elem
								.clone()
								.insertAfter(elem);

						_form.appendChild(this);
						stack.push( [ elem, spacer ] );
					});

			// first match form
			form[0].reset();
			form.triggerHandler('reset');

			// matched elements
			_form.reset();

			$.each( stack, function( i, data ){
				var elem = data[0],
					spacer = data[1];

				elem
					.insertBefore(spacer)
					.triggerHandler('reset');
				spacer.remove();
			});

			return this;
		},
		// clear all selected forms or formInputs
		formClear: function(){
			this.filter(':file')
				.formReset();

			clearValues( this.formInput() );
			clearValues( this.filter(':input') );

			return this;
		}
	});

});