/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.xml
 */

jQuery.plugin( 'jQuery.xml', function( $, undefined ){

	var createMap = {
			1: 'createElement',
			3: 'createTextNode',
			4: 'createCDATASection',
			8: 'createComment'
		},
		// original jQuery.fn.html
		_fn_html = jQuery.fn.html,
		// original jQuery.clean
		_clean = jQuery.clean,
		// clean for XML
		cleanXML = function( xmlString, context, fragment ){
			xmlString = "<root>"+xmlString+"</root>";

			var xmlDoc,
				appendNodes = function( target, copy, level ){
					var type = copy.nodeType,
						create = createMap[ type ],
						clone;

					if( level > 1 && create ){
						clone = target.appendChild( context[ create ]( type===1 ? copy.nodeName : copy.text || copy.nodeValue || '' ) );

						if( type===1 ){
							$.each( copy.attributes || [], function( i, attr ){
								var name = attr.name;
								// add attribute, using the jQuery.props map
								clone.setAttribute( $.props[ name ] || name, attr.value );
							});

							target = clone;
						}
					}

					$.each( copy.childNodes || [], function( i, child ){
						appendNodes( target, child, level+1 );
					});
				};

			// create XML document from string
			if( DOMParser !== undefined ){
				xmlDoc = (new DOMParser()).parseFromString( xmlString, "application/xml" );
			}
			else{
				var parsers = ["MSXML2.DOMDocument","MSXML.DOMDocument","Microsoft.XMLDOM"];

				for( var i=0,l=parsers.length ; i<l && !xmlDoc ; i++ ){
					try{
						xmlDoc = new ActiveXObject( parsers[ i ] );
						xmlDoc.loadXML(xmlString);
					}
					catch(e){
					}
				}
			}

			// clone XML document to fragment
			if( xmlDoc )
				appendNodes( fragment, xmlDoc.documentElement, 1 );

			return fragment;
		};

	// new jQuery.clean
	jQuery.clean = function( elems, context, fragment, scripts ){
		if( $.isXMLDoc(context) && elems.length === 1 && typeof elems[0] === "string" ){
			var ret = cleanXML( elems[0], context, fragment );
			return ret;
		}
		else{
			return _clean.call( $, elems, context, fragment, scripts );
		}
	};

	$.fn.extend({
		// redirect for get innerXML, need for .domManip() if value is a function
		html: function( value ){
			return $.isXMLDoc(this[0]) && value===undefined ?
				this.xml() :
				_fn_html.call( this, value );
		},
		// set + get innerXML
		xml: function( value ) {
			var firstNode = this[0] || {},
				ret = this,
				serializer;

			if( !$.isXMLDoc(firstNode) ){
				jQuery.error("TypeError: Result of expression 'this.xml' [undefined] is only supported for XML documents.");
				return;
			}

			if( value===undefined ){
				ret = '';

				if( XMLSerializer!==undefined )
					serializer = new XMLSerializer();

				$.each( firstNode.childNodes || [], function( i, node ){
					ret += serializer ?
						serializer.serializeToString(node) :
						node.xml;
				});
			}
			else{
				this.each(function( i, node ){
					var elem = $(this),
						oldXML = elem.xml(),
						newXML = value;

					elem.empty()
						.append( $( $.isFunction(newXML) ? newXML(i, oldXML) : newXML, this ) );
				});
			}

			return ret;
		}
	});

});