use strict;
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Presentation::API::PresentationInstaller;

sub Main {
    plan tests => 18;

    my $dbi = GetCurrentDBHandle();
    my $StoreName = $dbi->storeName;
    my $Installer = DE_EPAGES::Presentation::API::PresentationInstaller->new(
        CartridgeDirectory => 'DE_EPAGES/Presentation',
        StoreName => $StoreName
    );

    my $System = LoadRootObject();
    my $hOrgValues = $System->get(['DocRoot', 'WebRoot']);

    # test _convertDocRoot
    eval {
        $System->set({'WebRoot' => '/WebRoot', 'DocRoot' => '/WebRoot/Doc'});

        cmp_ok( $System->get('WebRoot'), 'eq', '/WebRoot', "default WebRoot" );
        ##no critic ( ProhibitPrivateSubroutineUse)
        cmp_ok( $System->get('DocRoot'), 'eq', '/WebRoot/Doc', "default DocRoot" );
        cmp_ok( $Installer->_convertDocRoot('Doc'), 'eq', 'Doc', "default Doc" );
        cmp_ok( $Installer->_convertDocRoot('Doc/Help'), 'eq', 'Doc/Help', "default Doc/Help" );
        cmp_ok( $Installer->_convertDocRoot('Icons'), 'eq', 'Icons', "default directory" );
        cmp_ok( $Installer->_convertDocRoot('Icons/heavytraffic_ico.gif'), 'eq', 'Icons/heavytraffic_ico.gif', "default file" );
        ## use critic

        $System->set({'WebRoot' => '/WebRoot', 'DocRoot' => '/WebRoot/MyDoc'});

        cmp_ok( $System->get('WebRoot'), 'eq', '/WebRoot', "custom WebRoot" );
        cmp_ok( $System->get('DocRoot'), 'eq', '/WebRoot/MyDoc', "custom DocRoot" );
        ##no critic ( ProhibitPrivateSubroutineUse)
        cmp_ok( $Installer->_convertDocRoot('Doc'), 'eq', 'MyDoc', "custom Doc" );
        cmp_ok( $Installer->_convertDocRoot('Doc/Help'), 'eq', 'MyDoc/Help', "custom Doc/Help" );
        cmp_ok( $Installer->_convertDocRoot('Icons'), 'eq', 'Icons', "custom directory" );
        cmp_ok( $Installer->_convertDocRoot('Icons/heavytraffic_ico.gif'), 'eq', 'Icons/heavytraffic_ico.gif', "custom file" );
        ## use critic

        $System->set({'WebRoot' => '/WebRoot', 'DocRoot' => '/External/Doc'});

        cmp_ok( $System->get('WebRoot'), 'eq', '/WebRoot', "external WebRoot" );
        cmp_ok( $System->get('DocRoot'), 'eq', '/External/Doc', "external DocRoot" );
        ##no critic ( ProhibitPrivateSubroutineUse)
        cmp_ok( $Installer->_convertDocRoot('Doc'), 'eq', 'Doc', "external Doc" );
        cmp_ok( $Installer->_convertDocRoot('Doc/Help'), 'eq', 'Doc/Help', "external Doc/Help" );
        cmp_ok( $Installer->_convertDocRoot('Icons'), 'eq', 'Icons', "external directory" );
        cmp_ok( $Installer->_convertDocRoot('Icons/heavytraffic_ico.gif'), 'eq', 'Icons/heavytraffic_ico.gif', "external file" );
        ## use critic
    };

    $System->set( $hOrgValues );
}

RunScript( Debug => 1, Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
